#include "StdAfx.h"
#include "util.h"
#include "iniio.h"
#include "FormatMessageSetting.h"
using namespace Util;

#define SECTION_FORMATMESSAGE	TEXT("FormatMessage")
#define KEY_ERRCODEFROM			TEXT("ErrCodeFrom")
#define KEY_ERRCODETO			TEXT("ErrCodeTo")
#define KEY_SEARCH				TEXT("Search")
#define KEY_FINDMSG				TEXT("FindMsg")
#define KEY_DLLLIST				TEXT("DllList")
#define DLLLIST_DELIMIT			TEXT("|")

CFormatMessageSetting::CFormatMessageSetting(void) :
	m_dwErrCodeFrom(1)
	, m_dwErrCodeTo(12500)
	, m_nSearch(1)
	, m_strFindMsg(TEXT(""))
{
	m_strDlls.Add(TEXT("WININET.DLL"));
}

void CFormatMessageSetting::LoadSetting(LPCTSTR pszIni)
{
	LPTSTR	psz = GetPrivateProfileSectionByHeapAlloc(SECTION_FORMATMESSAGE, 1024, pszIni);
	if(! psz)
	{
		return;
	}
	CIniSectionParser	parser(psz);
	m_dwErrCodeFrom	= parser.GetIntValue(KEY_ERRCODEFROM, m_dwErrCodeFrom);
	m_dwErrCodeTo	= parser.GetIntValue(KEY_ERRCODETO, m_dwErrCodeTo);
	m_nSearch		= parser.GetIntValue(KEY_SEARCH, m_nSearch);
	m_strFindMsg	= parser.GetStringValue(KEY_FINDMSG, m_strFindMsg);
	LPCTSTR	pszDllList = parser.GetStringValue(KEY_DLLLIST, NULL);
	if(pszDllList)
	{
		m_strDlls.RemoveAll();
		StrToken	tok(const_cast<LPTSTR>(pszDllList));
		while(LPCTSTR pszDll = tok(DLLLIST_DELIMIT))
		{
			m_strDlls.Add(pszDll);
		}
	}
	HeapFree(GetProcessHeap(), 0, psz);
}

void CFormatMessageSetting::SaveSetting(LPCTSTR pszIni)
{
	CString	strDllList;
	INT_PTR	nCount = m_strDlls.GetSize();
	for(INT_PTR i = 0; i < nCount; ++i)
	{
		if(! strDllList.IsEmpty())	strDllList.Append(DLLLIST_DELIMIT);
		strDllList.Append(m_strDlls.GetAt(i));
	}
	CIniSectionWriterEx<>	writer;
	writer.Write(KEY_ERRCODEFROM, m_dwErrCodeFrom);
	writer.Write(KEY_ERRCODETO, m_dwErrCodeTo);
	writer.Write(KEY_SEARCH, m_nSearch);
	writer.Write(KEY_FINDMSG, m_strFindMsg);
	writer.Write(KEY_DLLLIST, strDllList);
	writer.WriteIni(SECTION_FORMATMESSAGE, pszIni);
}
