/*
 * $Workfile: iniio.h $
 * Copyright (C) 2005 egrath.
 *
 * ini file read, write
 */
#pragma once
#include <windows.h>
#include <shlwapi.h>	// wvnsprintf, StrChr 
#include <map>
#pragma comment(lib, "shlwapi")
namespace Util {
struct KeyCompare
{
	inline bool operator()(const LPCTSTR& lhs, const LPCTSTR& rhs) const { return (lstrcmpi(lhs, rhs) < 0);  }
};
typedef std::map<LPCTSTR, LPCTSTR, KeyCompare>	ValueCollection;
typedef ValueCollection::value_type				KeyValue;
typedef ValueCollection::const_iterator			ValueCIterator;


// ----------------------------------------------------------------------------------
// CIniSectionParser
//
class CIniSectionParser
{
	ValueCollection	_valus;
	static KeyValue _Split(LPTSTR pszLine)
	{
		LPTSTR	pszEqual = StrChr(pszLine, TEXT('='));
		if(pszEqual)
		{
			*pszEqual = '\0';
			LPTSTR	pszValue = pszEqual+1;
			if(*pszValue == TEXT('"'))
			{
				++pszValue;
				int	nLen = lstrlen(pszValue);
				if(*(pszValue+nLen-1) == TEXT('"'))
				{
					*(pszValue+nLen-1) = '\0';
				}
			}
			return KeyValue(pszLine, pszValue);
		}
		return KeyValue(NULL, NULL);
	}
public:
	CIniSectionParser(LPTSTR psz)
	{
		Parse(psz);
	}
	LPCTSTR GetStringValue(LPCTSTR pszKey, LPCTSTR pszDefault) const
	{
		ValueCIterator	it = _valus.find(pszKey);
		if(it == _valus.end())
		{
			return pszDefault;
		}
		return it->second;
	}
	int GetIntValue(LPCTSTR pszKey, int nDefault) const
	{
		LPCTSTR	pszValue = GetStringValue(pszKey, NULL);
		if(! pszValue)
		{
			return nDefault;
		}
		return StrToInt(pszValue);
	}
	int Parse(LPTSTR psz)
	{
		int		nLoadLine = 0;
		LPTSTR	pszLine = psz, pszNextLine;
		try
		{
			while(*pszLine)
			{
				// set next line
				pszNextLine = pszLine + lstrlen(pszLine)+1;
	
				// split '=', and insert element
				KeyValue	split = _Split(pszLine);
				if(split.first)
				{
					if(_valus.insert(split).second)
					{
						++nLoadLine;
					}
				}

				// next line
				pszLine = pszNextLine;
			}
		}
		catch(...)
		{ 
			nLoadLine = -1;
		}
		return nLoadLine;
	}
};

// ----------------------------------------------------------------------------------
// CIniSectionWriter
//
class CIniSectionWriter
{
public:
	LPTSTR	m_pszBuf;
	int		m_cchBuf;
	int		m_cchWrited;
	CIniSectionWriter(LPTSTR pszBuf, int cchBuf) : m_pszBuf(pszBuf), m_cchBuf(cchBuf), m_cchWrited(0)
	{ }
	LPCTSTR GetData() const
	{
		return m_pszBuf;
	}
	BOOL Write(LPCTSTR pszKey, LPCTSTR pszVal)
	{
		if(pszVal)
		{
			int	cch = wnsprintf(m_pszBuf+m_cchWrited, m_cchBuf-m_cchWrited-2, TEXT("%s=%s"), pszKey, pszVal);
			if(cch < 0)
			{
				return FALSE;
			}
			m_cchWrited += cch+1;
		}
		return TRUE;
	}
	BOOL Write(LPCTSTR pszKey, long val)
	{
		TCHAR	szVal[14];
		wsprintf(szVal, TEXT("%d"), val);
		return Write(pszKey, szVal);
	}
	BOOL WriteFormat(LPCTSTR pszFmt, ...)
	{	
		va_list args;
		va_start(args, pszFmt);
		int	cch = wvnsprintf(m_pszBuf+m_cchWrited, m_cchBuf-m_cchWrited-2, pszFmt, args);
		va_end(args);
		if(cch < 0)
		{
			return FALSE;
		}
		m_cchWrited += cch+1;
		return TRUE;
	}
	BOOL WriteIni(LPCTSTR pszSection, LPCTSTR pszIni)
	{
		m_pszBuf[m_cchWrited] = '\0';
		return WritePrivateProfileSection(pszSection, (m_cchWrited ? m_pszBuf : NULL), pszIni);
	}
};

// ----------------------------------------------------------------------------------
// CIniSectionWriterEx
//
template< int cchGrow = 1024 >
class CIniSectionWriterEx
{
public:
	LPTSTR	m_pszBuf;
	int		m_cchBuf;
	int		m_cchWrited;
	CIniSectionWriterEx() : 
		m_pszBuf(NULL), m_cchBuf(0), m_cchWrited(0)
	{ 
		AllocBuf();
	}
	~CIniSectionWriterEx()
	{
		if(m_pszBuf)
		{
			HeapFree(GetProcessHeap(), 0, m_pszBuf);
		}
	}
	BOOL AllocBuf()
	{
		LPTSTR	pszAlloc = NULL;
		if(m_pszBuf)
		{
			pszAlloc = (LPTSTR)HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, m_pszBuf, (m_cchBuf+cchGrow)*sizeof(TCHAR));
		}
		else
		{
			pszAlloc = (LPTSTR)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, cchGrow*sizeof(TCHAR));
		}
		if(! pszAlloc)
		{
			return FALSE;
		}
		m_pszBuf = pszAlloc;
		m_cchBuf = static_cast<int>(HeapSize(GetProcessHeap(), 0, pszAlloc));
		return TRUE;
	}
	LPCTSTR GetData() const
	{
		return m_pszBuf;
	}
	int GetWritered() const
	{
		return m_cchWrited;
	}
	BOOL Write(LPCTSTR pszKey, LPCTSTR pszVal)
	{
		if(pszVal)
		{
			while(true)
			{
				int	cch = wnsprintf(m_pszBuf+m_cchWrited, m_cchBuf-m_cchWrited-2, TEXT("%s=%s"), pszKey, pszVal);
				if(cch < 0)
				{
					if(! AllocBuf())
					{
						return FALSE;
					}
					continue;
				}
				m_cchWrited += cch+1;
				break;
			}
		}
		return TRUE;
	}
	BOOL Write(LPCTSTR pszKey, long val)
	{
		TCHAR	szVal[14];
		wsprintf(szVal, TEXT("%d"), val);
		return Write(pszKey, szVal);
	}
	BOOL WriteFormat(LPCTSTR pszFmt, ...)
	{	
		while(true)
		{
			va_list args;
			va_start(args, pszFmt);
			int	cch = wvnsprintf(m_pszBuf+m_cchWrited, m_cchBuf-m_cchWrited-2, pszFmt, args);
			va_end(args);
			if(cch < 0)
			{
				if(! AllocBuf())
				{
					return FALSE;
				}
				continue;
			}
			m_cchWrited += cch+1;
			break;
		}
		return TRUE;
	}
	BOOL WriteIni(LPCTSTR pszSection, LPCTSTR pszIni)
	{
		return WritePrivateProfileSection(pszSection, (m_cchWrited ? m_pszBuf : NULL), pszIni);
	}
};

//---------------------------------------------------------------------------------//
// get private profile section
//---------------------------------------------------------------------------------//
inline LPTSTR GetPrivateProfileSectionByHeapAlloc(LPCTSTR pszSection, DWORD cchGrow, LPCTSTR pszIni)
{
	// ǂݎ菈
	DWORD	cchBuf = cchGrow;
	LPTSTR	pszAlloc = (LPTSTR)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, cchBuf*sizeof(TCHAR));
	if(! pszAlloc)
	{
		return NULL;
	}
	DWORD	dwReaded;
	while(true)
	{
		cchBuf = static_cast<DWORD>(HeapSize(GetProcessHeap(), 0, pszAlloc)/sizeof(TCHAR));
		dwReaded = GetPrivateProfileSection(pszSection, pszAlloc, cchBuf, pszIni);
		if((cchBuf-2) == dwReaded)
		{
			cchBuf += cchGrow;
			LPTSTR	pszRealloc = (LPTSTR)HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, pszAlloc, cchBuf*sizeof(TCHAR));
			if(! pszRealloc)
			{
				HeapFree(GetProcessHeap(), 0, pszAlloc);
				return NULL;
			}
			pszAlloc = pszRealloc;
			continue;
		}
		break;
	}
	return pszAlloc;
}
}		// namespace Util
