#include "stdafx.h"
#include "FormatMessage.h"
#include "DllListDlg.h"
#include "MsgSearchDlg.h"
#include "util.h"

BOOL Util::FindFormatMessage(DWORD dwMessageId , const CStringArray& strDlls, OUT LPTSTR* ppMsg)
{
	ASSERT(ppMsg);
	BOOL	bRet = ::FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 
			NULL, 
			dwMessageId, 
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR)ppMsg, 
			0, 
			NULL);
	if(! bRet)
	{
		for(INT_PTR i = 0; i < strDlls.GetCount(); ++i)
		{
			HMODULE	hModule = LoadLibraryEx(strDlls.GetAt(i), NULL, DONT_RESOLVE_DLL_REFERENCES | LOAD_LIBRARY_AS_DATAFILE);
			if(hModule)
			{
				bRet = ::FormatMessage(
					FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_FROM_HMODULE, 
					hModule, 
					dwMessageId, 
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
					(LPTSTR)ppMsg, 
					0, 
					NULL);
				FreeLibrary(hModule);
				if(bRet)
				{
					break;
				}
			}
		}
	}
	return bRet;
}

BOOL Util::DllListDlg(CWnd* pParent, IN OUT CStringArray& strDlls)
{
	CDllListDlg	dlg(pParent);
	dlg.m_strDlls.Copy(strDlls);
	if(IDOK == dlg.DoModal())
	{
		strDlls.Copy(dlg.m_strDlls);
		return TRUE;
	}
	return FALSE;
}

BOOL Util::MsgSearchDlg(CWnd* pParent, IN OUT MsgDlgParam& mdp)
{
	CMsgSearchDlg	dlg(pParent);
	dlg.m_hwndErrList = mdp.hwndErrList;
	dlg.m_strMsg = mdp.strMsg;
	dlg.m_nSearch = mdp.nSearch;
	dlg.DoModal();
	if(dlg.m_bDoSearch)
	{
		mdp.strMsg = dlg.m_strMsg;
		mdp.nSearch = dlg.m_nSearch;
		return TRUE;
	}
	return FALSE;
}

BOOL Util::SetClipboardText(HWND hwnd, LPCTSTR psz)
{
	int		nTextLen = lstrlen(psz)+1;
	HGLOBAL	hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, nTextLen*sizeof(TCHAR));
	if(! hGlobal)
	{
		return FALSE;
	}
	LPVOID	lpText = ::GlobalLock(hGlobal);
	if(! lpText)
	{
		::GlobalFree(hGlobal);
		return FALSE;
	}
	memcpy(lpText, psz, nTextLen*sizeof(TCHAR));
	::GlobalUnlock(hGlobal);
	if(! ::OpenClipboard(hwnd))
	{
		::GlobalFree(hGlobal);
		return FALSE;
	}
	UINT	fmt = 0;
#if defined(_UNICODE) || defined(UNICODE)
		fmt = CF_UNICODETEXT;
#else
		fmt = CF_TEXT;
#endif	
	::EmptyClipboard();
	::SetClipboardData(fmt, hGlobal);
	::CloseClipboard();
	return TRUE;
}

LPTSTR Util::StrRemove(LPTSTR psz, LPCTSTR pszRemove)
{
	LPTSTR	pszNow = psz;
	while((pszNow = StrPBrk(pszNow, pszRemove)))
	{
		memmove(pszNow, pszNow+1, (lstrlen(pszNow+1)+1)*sizeof(TCHAR));
	}
	return psz;
}
